import marimo

__generated_with = "0.18.4"
app = marimo.App()


@app.cell
def _():
    import sys

    import pandas as pd
    import marimo as mo

    sys.path.append("../")
    import mustaching as ms

    # magic command not supported in marimo; please file an issue to add support
    # %load_ext autoreload
    # '%autoreload 2' command supported automatically in marimo
    return mo, ms, pd


@app.cell(hide_code=True)
def _(mo):
    mo.md(r"""
    # Load transactions
    """)
    return


@app.cell
def _(ms):
    # Invent some sample transactions.
    transactions = ms.create_transactions("2021-01-01", "2021-12-31")

    # Alternatively, upload your own transactions as say 'my_transactions.csv'
    # transactions = ms.read_transactions('my_transactions.csv')

    transactions.head(10)
    return (transactions,)


@app.cell(hide_code=True)
def _(mo):
    mo.md(r"""
    # Summarize and plot transactions
    """)
    return


@app.cell
def _(mo, ms, pd, transactions):
    def run_demo_1(transactions: pd.DataFrame):
        summary = ms.summarize(
            transactions, freq="QS"
        )  # dictionary of kind -> DataFrame
        for k, v in summary.items():
            mo.output.append(k)
            mo.output.append(v)

        plot = ms.plot(summary, currency="$")
        for k, v in plot.items():  # dictionary of kind -> Plotly figure
            mo.output.append(k)
            mo.output.append(v)

    run_demo_1(transactions)
    return


@app.cell(hide_code=True)
def _(mo):
    mo.md(r"""
    # For transactions without categories we get the following
    """)
    return


@app.cell
def _(mo, ms, pd, transactions):
    def run_demo_2(transactions: pd.DataFrame):
        summary = ms.summarize(transactions.drop("category", axis="columns"), freq="QS")
        for k, v in summary.items():
            mo.output.append(k)
            mo.output.append(v)
        plot = ms.plot(summary, currency="$")
        for k, v in plot.items():  # dictionary of kind -> Plotly figure
            mo.output.append(k)
            mo.output.append(v)

    run_demo_2(transactions)
    return


@app.cell
def _():
    return


if __name__ == "__main__":
    app.run()



import pandas as pd
import pandera as pa
import pytest

from .context import mustaching
from mustaching import main as m


def test_create_transactions():
    t = m.create_transactions("2016-12-01", "2017-03-01")
    assert isinstance(t, pd.DataFrame)
    assert set(t.columns) == {"date", "amount", "description", "category", "comment"}
    assert isinstance(t["date"].iat[0], pd.Timestamp)


def test_validate_transactions():
    f = pd.DataFrame(
        {
            "date": ["2020-12-31"],
            "amount": [69],
        }
    )
    m.validate_transactions(f)

    f = pd.DataFrame(
        {
            "Date": ["2020-12-31"],
            "amount": [69],
        }
    )
    with pytest.raises(pa.errors.SchemaError):
        m.validate_transactions(f)

    f = pd.DataFrame(
        {
            "date": [2020],
            "amount": [69],
        }
    )
    with pytest.raises(pa.errors.SchemaError):
        m.validate_transactions(f)


def test_insert_repeating():
    t = m.create_transactions("2017-01-01", "2017-03-01")
    desc = "oh no!"
    f = m.insert_repeating(t, -100, "MS", desc)
    assert f.shape[0] == t.shape[0] + 3
    assert f["amount"].sum() == t["amount"].sum() - 300
    assert f["category"].dtype == "category"


def test_summarize():
    t = m.create_transactions("2017-01-01", "2017-12-31")
    s = m.summarize(t, freq="QS", decimals=None)

    assert set(s.keys()) == {
        "by_none",
        "by_period",
        "by_category",
        "by_category_and_period",
    }

    assert set(s["by_none"].columns) == {
        "start_date",
        "end_date",
        "income",
        "expense",
        "balance",
        "savings_pc",
    }

    assert set(s["by_period"].columns) == {
        "date",
        "income",
        "expense",
        "balance",
        "savings_pc",
        "cumulative_income",
        "cumulative_balance",
        "cumulative_savings_pc",
    }

    assert set(s["by_category"].columns) == {
        "category",
        "income",
        "expense",
        "balance",
        "income_to_total_income_pc",
        "expense_to_total_income_pc",
        "expense_to_total_expense_pc",
        "daily_avg_balance",
        "weekly_avg_balance",
        "monthly_avg_balance",
        "yearly_avg_balance",
    }
    assert s["by_category"].income_to_total_income_pc.sum() == pytest.approx(100)
    assert s["by_category"].expense_to_total_expense_pc.sum() == pytest.approx(100)

    assert set(s["by_category_and_period"].columns) == {
        "date",
        "category",
        "income",
        "expense",
        "balance",
        "income_to_period_income_pc",
        "expense_to_period_income_pc",
        "expense_to_period_expense_pc",
    }
    for __, group in s["by_category_and_period"].groupby("date"):
        assert group.income_to_period_income_pc.sum() == pytest.approx(100)
        assert group.expense_to_period_expense_pc.sum() == pytest.approx(100)

    # Unused categories should by dropped
    s = m.summarize(t.iloc[-2:])
    assert (
        s["by_category"].category.cat.categories.size
        == s["by_category"].category.nunique()
    )

    s = m.summarize(t.drop(["category"], axis="columns"))
    assert s["by_category"].empty
    assert s["by_category_and_period"].empty


def test_plot():
    t = m.create_transactions("2017-01-01", "2017-12-31")

    summary = m.summarize(t, freq="MS")
    p = m.plot(summary, currency="$")
    assert set(p.keys()) == {"by_category", "by_category_and_period"}

    summary = m.summarize(t.drop("category", axis="columns"), freq="MS")
    p = m.plot(summary, currency="$")
    assert set(p.keys()) == {"by_none", "by_period"}
